__author__ = 'Nadeem Douba'
__copyright__ = 'Copyright 2015, Canari Project'
__credits__ = []

__license__ = 'GPLv3'
__version__ = '3.3.0'
__maintainer__ = 'Nadeem Douba'
__email__ = 'ndouba@gmail.com'
__status__ = 'Development'

__all__ = [
    'maltego',
    'resources',
    'utils',
    'config',
    'easygui',
    'framework',
    'mode',
    'resource',
    'tas',
    'entrypoints'
]

